#include "liquid.h"
#include <qlayout.h>
#include <kapp.h>
#include <ksimpleconfig.h>
#include <klocale.h>
#include <qvbuttongroup.h>
#include <qvgroupbox.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qtabwidget.h>
#include <qslider.h>
#include <qlabel.h>
#include <kcolorbutton.h>
#include <qsettings.h>
#include <kipc.h>

void LiquidConfig::save()
{
    QSettings *settings = new QSettings;
    int id = btnGroup->id(btnGroup->selected());
    settings->writeEntry("/qt/LiquidMenus/Type", id);
    settings->writeEntry("/qt/LiquidMenus/ShadowText", shadowBtn->isOn());
    if(id == Custom){
        settings->writeEntry("/qt/LiquidMenus/Color", (int)colorBtn->color().rgb());
        settings->writeEntry("/qt/LiquidMenus/TextColor", (int)fgBtn->color().rgb());
        settings->writeEntry("/qt/LiquidMenus/Opacity", opacitySlider->value());
    }

    settings->writeEntry("/qt/Liquid/BgStipple", stippleBgBtn->isOn());
    settings->writeEntry("/qt/Liquid/PanelBgStipple", stipplePanelBtn->isOn());
    settings->writeEntry("/qt/Liquid/StippleContrast", stippleContrastSlider->value());
    settings->writeEntry("/qt/Liquid/ReverseBtnColor", reverseColorBtn->isOn());
    settings->writeEntry("/qt/Liquid/CustomPanelColor", usePanelCustomBtn->isOn());
    if(usePanelCustomBtn->isOn()){
        settings->writeEntry("/qt/Liquid/PanelColor", (int)panelColorBtn->color().rgb());
    }
    settings->writeEntry("/qt/Liquid/ToolButtonFrame", useTbFrameBtn->isOn());
    delete settings; // force sync
    KIPC::sendMessageAll(KIPC::StyleChanged, 0);
    emit changed(false);
}


void LiquidConfig::load()
{
    QSettings settings;

    int id = settings.readNumEntry("/qt/LiquidMenus/Type", StippledBg);
    btnGroup->setButton(id);
    colorBtn->setColor(QColor(settings.readNumEntry("/qt/LiquidMenus/Color",
                                                    (int)colorGroup().button().rgb())));
    fgBtn->setColor(QColor(settings.readNumEntry("/qt/LiquidMenus/TextColor",
                                                 (int)colorGroup().text().rgb())));
    opacitySlider->setValue(settings.readNumEntry("/qt/LiquidMenus/Opacity",
                                                  10));
    shadowBtn->setChecked(settings.readBoolEntry("/qt/LiquidMenus/ShadowText",
                                                 true));

    stippleBgBtn->setChecked(settings.readBoolEntry("/qt/Liquid/BgStipple",
                                                    true));
    stipplePanelBtn->setChecked(settings.readBoolEntry("/qt/Liquid/PanelBgStipple",
                                                       true));
    reverseColorBtn->setChecked(settings.readBoolEntry("/qt/Liquid/ReverseBtnColor",
                                                       true));
    stippleContrastSlider->setValue(settings.readNumEntry("/qt/Liquid/StippleContrast",
                                                          3));
    usePanelCustomBtn->setChecked(settings.readBoolEntry("/qt/Liquid/CustomPanelColor",
                                                         false));
    if(usePanelCustomBtn->isOn())
        panelColorBtn->setColor(QColor(settings.readNumEntry("/qt/Liquid/PanelColor",
                                                             (int)colorGroup().button().rgb())));
    else
        panelColorBtn->setColor(colorGroup().button().rgb());

    useTbFrameBtn->setChecked(settings.readBoolEntry("/qt/Liquid/ToolButtonFrame",
                                                     true));
    emit changed(false);
}

void LiquidConfig::defaults()
{
    load();
    emit changed(true);
}

void LiquidConfig::slotBtnGroupClicked(int id)
{
    editGroup->setEnabled(id == Custom);
    emit changed(true);
}

void LiquidConfig::slotColorOptionChanged()
{
    emit changed(true);
}

void LiquidConfig::slotPanelColorChanged(const QColor &)
{
    emit changed(true);
}

void LiquidConfig::slotSliderChanged(int)
{
    emit changed(true);
}

void LiquidConfig::slotUsePanelCustomClicked(bool on)
{
    panelColorBtn->setEnabled(on);
    emit changed(true);
}

LiquidConfig::LiquidConfig(QWidget *parent, const char *name)
    : KCModule(parent, name)
{
    QVBoxLayout *layout = new QVBoxLayout(this);
    QTabWidget *tabWidget = new QTabWidget(this);
    layout->addWidget(tabWidget);

    QWidget *page1 = new QWidget(tabWidget);
    layout = new QVBoxLayout(page1);
    layout->addSpacing(10);

    btnGroup = new QVButtonGroup(i18n("Menu style"), page1);
    (void)new QRadioButton(i18n("No translucency"), btnGroup);
    (void)new QRadioButton(i18n("Stippled, background color"), btnGroup);
    (void)new QRadioButton(i18n("Stippled, button color"), btnGroup);
    (void)new QRadioButton(i18n("Translucent stippled, background color"), btnGroup);
    (void)new QRadioButton(i18n("Translucent stippled, button color"), btnGroup);
    (void)new QRadioButton(i18n("Custom translucency"), btnGroup);
    connect(btnGroup, SIGNAL(clicked(int)), SLOT(slotBtnGroupClicked(int)));
    layout->addWidget(btnGroup);

    editGroup = new QGroupBox(i18n("Translucency editor"), page1);
    QGridLayout *editLayout = new QGridLayout(editGroup, 1, 1, 10);
    QLabel *colorLbl = new QLabel(i18n("Menu color:"), editGroup);
    colorBtn =  new KColorButton(editGroup);
    QLabel *fgLbl = new QLabel(i18n("Text color:"), editGroup);
    fgBtn =  new KColorButton(editGroup);

    QLabel *opacityLbl = new QLabel(i18n("Opacity:"), editGroup);
    opacitySlider = new QSlider(0, 20, 1, 10, Qt::Horizontal,
                                editGroup);
    connect(opacitySlider, SIGNAL(valueChanged(int)), this,
            SLOT(slotSliderChanged(int)));

    editLayout->addRowSpacing(0, 12);
    editLayout->addWidget(colorLbl, 1, 0);
    editLayout->addWidget(colorBtn, 1, 1);
    editLayout->addWidget(fgLbl, 1, 2);
    editLayout->addWidget(fgBtn, 1, 3);

    editLayout->addMultiCellWidget(opacityLbl, 2, 2, 0, 4);
    editLayout->addMultiCellWidget(opacitySlider, 3, 3, 0, 4);
    editLayout->setColStretch(4, 1);
    editLayout->setRowStretch(4, 1);

    layout->addWidget(editGroup);

    layout->addSpacing(16);
    shadowBtn = new QCheckBox(i18n("Use shadowed text"), page1);
    connect(shadowBtn, SIGNAL(clicked()), SLOT(slotColorOptionChanged()));
    layout->addWidget(shadowBtn);

    layout->addStretch(1);

    tabWidget->insertTab(page1, i18n("Menus"));

    QWidget *page2 = new QWidget(tabWidget);
    layout = new QVBoxLayout(page2);

    tabWidget->insertTab(page2, i18n("Colors, stipples, and Misc"));

    layout->addSpacing(10);

    QVGroupBox *stippleGrp = new QVGroupBox(i18n("Stipples"), page2);
    stippleBgBtn = new QCheckBox(i18n("Enable background stipple"),
                                 stippleGrp);
    connect(stippleBgBtn, SIGNAL(clicked()), this,
            SLOT(slotColorOptionChanged()));
    stipplePanelBtn = new QCheckBox(i18n("Enable panel stipple"),
                                    stippleGrp);
    connect(stipplePanelBtn, SIGNAL(clicked()), this,
            SLOT(slotColorOptionChanged()));

    QHBox *contrastBox = new QHBox(stippleGrp);
    QLabel *contrastLbl = new QLabel(i18n("Stipple contrast:"), contrastBox);
    stippleContrastSlider = new QSlider(1, 6, 1, 3, Qt::Horizontal,
                                        contrastBox);
    connect(stippleContrastSlider, SIGNAL(valueChanged(int)), this,
            SLOT(slotSliderChanged(int)));
    contrastBox->setStretchFactor(stippleContrastSlider, 1);
    layout->addWidget(stippleGrp);

    QVGroupBox *colorGrp = new QVGroupBox(i18n("Color usage"), page2);
    usePanelCustomBtn = new QCheckBox(i18n("Use custom panel color"), colorGrp);
    connect(usePanelCustomBtn, SIGNAL(toggled(bool)), this,
            SLOT(slotUsePanelCustomClicked(bool)));
    QHBox *colorBox = new QHBox(colorGrp);
    QLabel *panelColorLbl = new QLabel(i18n("Custom color:"), colorBox);
    panelColorBtn = new KColorButton(colorBox);
    connect(panelColorBtn, SIGNAL(changed(const QColor &)), this,
            SLOT(slotPanelColorChanged(const QColor &)));
    colorBox->setStretchFactor(new QWidget(colorBox), 1);
    colorGrp->addSpace(16);
    reverseColorBtn = new QCheckBox(i18n("Use background, not button, color for unpressed buttons"),
                                    colorGrp);
    connect(reverseColorBtn, SIGNAL(clicked()), this,
            SLOT(slotColorOptionChanged()));
    layout->addWidget(colorGrp);

    QVGroupBox *miscGrp = new QVGroupBox(i18n("Misc"), page2);
    useTbFrameBtn = new QCheckBox(i18n("Draw toolbar button frames"),
                                  miscGrp);
    connect(useTbFrameBtn, SIGNAL(clicked()),  this,
            SLOT(slotColorOptionChanged()));
    layout->addWidget(miscGrp);


    layout->addStretch(1);

    load();
    editGroup->setEnabled(btnGroup->id(btnGroup->selected()) == Custom);
    panelColorBtn->setEnabled(usePanelCustomBtn->isOn());
}

LiquidConfig::~LiquidConfig()
{
    ;
}



#include "liquid.moc"
















